/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

#ifndef DUNE_ASH_CURSOR_HH
#define DUNE_ASH_CURSOR_HH

namespace Dune
{

  namespace Ash
  {

    // MouseCursor
    // -----------

    class MouseCursor
    {
      typedef MouseCursor This;

    public:
      explicit MouseCursor ( const std::string &fileName );
      ~MouseCursor ();

    private:
      MouseCursor ( const This & );
      const This &operator= ( const This & );

      friend void setCursor ( const MouseCursor &cursor );

      SDL_Cursor *cursor_;
    };



    // Auxiliary Functions
    // -------------------

    void setCursor ( const MouseCursor &cursor );

    void hideCursor ();
    void showCursor ();

  } // namespace Ash

} // namespace Dune

#endif // #ifndef DUNE_ASH_CURSOR_HH
