/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

#ifndef DUNE_ASH_OPENGL_DISPLAYLIST_HH
#define DUNE_ASH_OPENGL_DISPLAYLIST_HH

#include <cassert>

#include <dune/ash/opengl/error.hh>
#include <dune/ash/opengl/opengl.hh>

namespace Dune
{

  namespace Ash
  {

    // DisplayList
    // -----------

    struct DisplayList
    {
      DisplayList ()
      {
        allocate();
      }

      template< class Functor >
      explicit DisplayList ( Functor functor )
      {
        allocate();
        compile( functor );
      }

      ~DisplayList ()
      {
        glDeleteLists( handle_, 1 );
        assert( glGetError() == GL_NO_ERROR );
      }

      void operator() () const { glCallList( handle_ ); }

      template< class Functor >
      void compile ( Functor functor )
      {
        assert( glGetError() == GL_NO_ERROR );
        glNewList( handle_, GL_COMPILE );
        functor();
        DUNE_ASH_CHECK_GLERROR;
        glEndList();
        DUNE_ASH_CHECK_GLERROR;
      }

    private:
      void allocate ()
      {
        handle_ = glGenLists( 1 );
        if( handle_ == GLuint( 0 ) )
        {
          DUNE_ASH_CHECK_GLERROR;
          DUNE_THROW( GLError, "Unable to allocate display list." );
        }
      }

      // forbid copying and assignment
      DisplayList ( const DisplayList & );
      const DisplayList &operator= ( const DisplayList & );

      GLuint handle_;
    };

  } // namespace Ash

} // namespace Dune

#endif // #ifndef DUNE_ASH_OPENGL_DISPLAYLIST_HH
