/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

#ifndef DUNE_ASH_TEXT_FONT_HH
#define DUNE_ASH_TEXT_FONT_HH

#include <SDL_ttf.h>

#include <dune/common/exceptions.hh>
#include <dune/common/fvector.hh>

#include <dune/ash/opengl/texture.hh>

namespace Dune
{

  namespace Ash
  {

    // TrueTypeFont
    // ------------

    class TrueTypeFont
    {
      typedef TrueTypeFont ThisType;

    public:
      TrueTypeFont ( const std::string &file, int size );
      ~TrueTypeFont ();

      std::string family () const { return TTF_FontFaceFamilyName( font_ ); }

      bool fixedWidth () const { return TTF_FontFaceIsFixedWidth( font_ ); }

      int lineSkip () const { return TTF_FontLineSkip( font_ ); }

      int maxAscent () const { return TTF_FontAscent( font_ ); }
      int maxDescent () const { return TTF_FontDescent( font_ ); }
      int maxHeight () const { return TTF_FontHeight( font_ ); }

      bool kerning () const { return TTF_GetFontKerning( font_ ); }
      void kerning ( bool enable ) { TTF_SetFontKerning( font_, enable ); }

      int style () const { return TTF_GetFontStyle( font_ ); }
      void style ( int style ) { TTF_SetFontStyle( font_, style ); }

      FieldVector< int, 2 > size ( const std::string &text ) const;

      GLfloat render ( const std::string &text, Texture &texture ) const;

    private:
      // prohibit copying and assignment
      TrueTypeFont ( const ThisType & );
      const ThisType &operator= ( const ThisType & );

      TTF_Font *font_;
    };



    // Implementation of TrueTypeFont
    // ------------------------------

    inline FieldVector< int, 2 > TrueTypeFont::size ( const std::string &text ) const
    {
      FieldVector< int, 2 > size;
      TTF_SizeUTF8( font_, text.c_str(), &size[ 0 ], &size[ 1 ] );
      return size;
    }

  } // namespace Ash

} // namespace Dune

#endif // #ifndef DUNE_ASH_TEXT_FONT_HH
