/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

#ifndef DUNE_ASH_TEXTURES_ARROW_HH
#define DUNE_ASH_TEXTURES_ARROW_HH


#include <dune/ash/fvector.hh>
#include <dune/ash/opengl/opengl.hh>


namespace Dune
{

  namespace Ash
  {

    // Arrow
    // -----

    struct Arrow
    {
      // displays the arrow used for the vector field visualization
      //
      // The arrow lives in the reference element [0,1]^2 and points from left to
      // right.

      Arrow( const GLdouble bodyRightWidth = 0.15 )
      : bodyRightWidth_( bodyRightWidth )
      {}

      void operator() ()
      {
        const GLdouble bodyLeftMargin = .1;
        const GLdouble bodyLeftWidth = .2;
        const GLdouble bodyInHeadOverlay = .01;
        const GLdouble headWidth = .4;
        const GLdouble headRightMargin = .1;

        glTranslate( -.5, -.5, 0. );

        glBegin( GL_QUADS );
        {
          glVertex( bodyLeftMargin, .5 - bodyLeftWidth/2. );
          glVertex( .5 + bodyInHeadOverlay, .5 - bodyRightWidth_/2. );
          glVertex( .5 + bodyInHeadOverlay, .5 + bodyRightWidth_/2. );
          glVertex( bodyLeftMargin, .5 + bodyLeftWidth/2. );
        }
        glEnd();

        glBegin( GL_TRIANGLES );
        {
          glVertex( .5, .5 - headWidth/2. );
          glVertex( 1. - headRightMargin, .5 );
          glVertex( .5, .5 + headWidth/2. );
        }
        glEnd();
      }

    private:
      const GLdouble bodyRightWidth_;

    };

  } // namespace Ash

} // namespace Dune


#endif //#ifndef DUNE_ASH_TEXTURES_ARROW_HH
