/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

// vim: set expandtab ts=2 sw=2 sts=2:
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <dune/fem/io/parameter.hh>

#include <dune/ash/widget/button.hh>

namespace Dune
{

  namespace Ash
  {

    void ButtonWidget::draw ()
    {
      glBegin( GL_QUADS );
      {
        glColor( background_ );
        glVertex( 0.0f, 0.0f );
        glVertex( 1.0f, 0.0f );
        glVertex( 1.0f, 1.0f );
        glVertex( 0.0f, 1.0f );
      }
      glEnd();

      glMatrixMode( GL_MODELVIEW );
      glPushMatrix();
      glTranslate( 0.1f, 0.1f, 0.0f );
      glScale( 0.8f, 0.8f, 1.0f );
      displayList_();
      glPopMatrix();

      glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
      glLineWidth( lineWidth_ );
      glBegin( GL_QUADS );
      {
        glColor( lineColor_ );
        glVertex( 0.0f, 0.0f );
        glVertex( 1.0f, 0.0f );
        glVertex( 1.0f, 1.0f );
        glVertex( 0.0f, 1.0f );
      }
      glEnd();
      glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
    }


    void ButtonWidget::mouseMove ( const Position &x, const Position &dx, Ticks ticks )
    {}


    void ButtonWidget::mouseButtonDown ( const Position &x, Ticks ticks )
    {
      pressed_ = true;
    }


    void ButtonWidget::mouseButtonUp ( const Position &x, Ticks ticks )
    {
      if( pressed_ && event_ )
        event_->executeEvent();
      pressed_ = false;
    }


    void ButtonWidget::initialize ()
    {
      pressed_ = false;
      background_ = Parameter::getValue< Color >( "ash.button.background", makeFieldVector( 1, 1, 1 ) );
      lineColor_ = Parameter::getValue< Color >( "ash.button.linecolor", makeFieldVector( 0, 0, 0 ) );
      lineWidth_ = Parameter::getValue< GLfloat >( "ash.button.linewidth", 3.0f );
    }

  } // namespace Ash

} // namespace Dune
