/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

// vim: set expandtab ts=2 sw=2 sts=2:
#ifndef DUNE_ASH_WIDGET_BUTTON_HH
#define DUNE_ASH_WIDGET_BUTTON_HH

#include <dune/common/nullptr.hh>

#include <dune/ash/fvector.hh>
#include <dune/ash/opengl/displaylist.hh>
#include <dune/ash/opengl/opengl.hh>
#include <dune/ash/widget/event.hh>
#include <dune/ash/widget/widget.hh>

namespace Dune
{

  namespace Ash
  {

    class ButtonWidget
    : public Dune::Ash::Widget
    {
      typedef ButtonWidget ThisType;

    public:
      typedef FieldVector< GLfloat, 3 > Color;

      template< class Functor >
      explicit ButtonWidget ( Event *event, Functor functor )
      : event_( event ),
        displayList_( functor )
      {
        initialize();
      }

      template< class Functor >
      explicit ButtonWidget ( Functor functor )
      : event_( nullptr ),
        displayList_( functor )
      {
        initialize();
      }

      void draw ();

      void mouseMove ( const Position &x, const Position &dx, Ticks ticks );

      void mouseButtonDown ( const Position &x, Ticks ticks );

      void mouseButtonUp ( const Position &x, Ticks ticks );

      template< class Functor >
      void setIcon( Functor functor ) { displayList_.compile( functor ); }

      Event *setEvent ( Event *event );

    private:
      void initialize ();

      Event *event_;
      DisplayList displayList_;
      bool pressed_;
      Color background_;
      Color lineColor_;
      GLfloat lineWidth_;
    };



    // Implementation of ButtonWidget
    // ------------------------------

    inline Event *ButtonWidget::setEvent ( Event *event )
    {
      std::swap( event_, event );
      return event;
    }

  } // namespace Ash

} // namespace Dune

#endif // #ifndef DUNE_ASH_WIDGET_BUTTON_HH
