/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

#ifndef DUNE_ASH_WIDGET_COLLECTION_HH
#define DUNE_ASH_WIDGET_COLLECTION_HH

#include <list>

#include <dune/ash/widget/widget.hh>

namespace Dune
{

  namespace Ash
  {

    // CollectionWidget
    // ----------------

    class CollectionWidget
    : public Widget
    {
      typedef Widget Base;

      struct Child
      {
        Child ( const Position &o, const Position &s, Widget *w )
        : origin( o ), size( s ),
          widget( w )
        {}

        bool contains ( const Position &x ) const
        {
          bool contains = true;
          for( int i = 0; i < 2; ++i )
            contains &= (x[ i ] - origin[ i ] >= 0) & (x[ i ] - origin[ i ] < size[ i ]);
          return contains;
        }

        Position position ( const Position &x ) const
        {
          Position position;
          for( int i = 0; i < 2; ++i )
            position[ i ] = (x[ i ] - origin[ i ]) / size[ i ];
          return position;
        }

        Position origin, size;
        Widget *widget;
      };

      typedef std::list< Child > ChildList;

    public:
      CollectionWidget ();

      void draw ();

      void mouseMove ( const Position &x, const Position &dx, Ticks ticks );

      void mouseButtonDown ( const Position &x, Ticks ticks );
      void mouseButtonUp ( const Position &x, Ticks ticks );

      void tick ( Ticks ticks );
      
      void addChild ( const Position &origin, const Position &size, Widget *widget );
      void removeChild ( Widget *widget );

    private:
      ChildList::const_iterator findChild ( const Position &x ) const;

      ChildList childList_;
      ChildList::const_iterator active_;
    };

  } // namespace Ash

} // namespace Dune

#endif // #ifndef DUNE_ASH_WIDGET_SPLIT_HH
