/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

// vim: set expandtab ts=2 sw=2 sts=2:
#ifndef DUNE_ASH_WIDGET_FLOATINGTEXT_HH
#define DUNE_ASH_WIDGET_FLOATINGTEXT_HH

#include <iostream>
#include <map>

#include <dune/ash/fvector.hh>
#include <dune/ash/text/box.hh>
#include <dune/ash/text/fontmanager.hh>
#include <dune/ash/opengl/opengl.hh>
#include <dune/ash/opengl/displaylist.hh>
#include <dune/ash/opengl/texture.hh>
#include <dune/ash/widget/event.hh>
#include <dune/ash/widget/widget.hh>

namespace Dune
{

  namespace Ash
  {

    // FloatingTextWidget
    // ------------------

    class FloatingTextWidget
    : public Dune::Ash::Widget
    {
      typedef FloatingTextWidget ThisType;
      typedef std::map< std::string, Event * > EventMap;
      typedef std::list< ActiveBox > ActiveBoxList;

    public:
      typedef FieldVector< GLfloat, 3 > Color;

      // construction and destruction

      FloatingTextWidget ( GLfloat dpu, const FieldVector< GLfloat, 2 > &size, std::istream &input );
      FloatingTextWidget ( GLfloat dpu, const FieldVector< GLfloat, 2 > &size, const std::string &textPath, std::istream &input );
      FloatingTextWidget ( GLfloat dpu, const FieldVector< GLfloat, 2 > &size, const std::string &fileName );
      FloatingTextWidget ( GLfloat dpu, const FieldVector< GLfloat, 2 > &size, const std::string &textPath, const std::string &fileName );
      FloatingTextWidget ( GLfloat dpu, const FieldVector< GLfloat, 2 > &size );
      FloatingTextWidget ( const FieldVector< GLfloat, 2 > &dpu, const FieldVector< GLfloat, 2 > &size, const std::string &textPath, std::istream &input );
      FloatingTextWidget ( const FieldVector< GLfloat, 2 > &dpu, const FieldVector< GLfloat, 2 > &size, std::istream &input );
      FloatingTextWidget ( const FieldVector< GLfloat, 2 > &dpu, const FieldVector< GLfloat, 2 > &size, const std::string &textPath, const std::string &fileName );
      FloatingTextWidget ( const FieldVector< GLfloat, 2 > &dpu, const FieldVector< GLfloat, 2 > &size, const std::string &fileName );
      FloatingTextWidget ( const FieldVector< GLfloat, 2 > &dpu, const FieldVector< GLfloat, 2 > &size );

      ~FloatingTextWidget ();

      // widget interface

      void draw ();
      void mouseMove ( const Position &x, const Position &dx, Ticks ticks );
      void mouseButtonDown ( const Position &x, Ticks ticks );
      void mouseButtonUp ( const Position &x, Ticks ticks );

      // special methods

      void setText ( const std::string &textPath, std::istream &input );
      void setText ( std::istream &input );
      void setText ( const std::string &textPath, const std::string &fileName );
      void setText ( const std::string &fileName );

      Event *setEvent ( const std::string &id, Event *event );
      void deleteEvents ();

    private:
      void setupText();

      FontManager fontManager_;
      FieldVector< GLfloat, 2 > size_;
      Color background_;
      ParBox *parBox_;
      Texture texture_;
      DisplayList displayList_;
      EventMap events_;
      ActiveBoxList activeBoxes_;
      ActiveBoxList::const_iterator activeBox_;
    };

  } // namespace Ash

} // namespace Dune

#endif // #ifndef DUNE_ASH_WIDGET_FLOATINGTEXT_HH
