/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

#ifndef DUNE_ASH_WIDGET_SHRINK_HH
#define DUNE_ASH_WIDGET_SHRINK_HH

#include <dune/ash/fvector.hh>
#include <dune/ash/opengl/opengl.hh>
#include <dune/ash/widget/widget.hh>

namespace Dune
{

  namespace Ash
  {


    // ShrinkWidget
    // -----------

    template< Reference reference = Center>
    class ShrinkWidget
    : public Widget
    {
      typedef Widget Base;

    public:
      typedef FieldVector< GLfloat, 4 > Color;

      ShrinkWidget ( GLfloat width, GLfloat height, GLfloat alpha = 0.5f );
      ShrinkWidget ( GLfloat width, GLfloat height, Color bgColor );

      void draw ();

      void mouseMove ( const Position &x, const Position &dx, Ticks ticks );

      void mouseButtonDown ( const Position &x, Ticks ticks );
      void mouseButtonUp ( const Position &x, Ticks ticks );

      Widget *setChild ( Widget *child )
      {
        std::swap( child, child_ );
        return child;
      }

      void ticks( Ticks ticks ) { if( child_ ) child_->tick( ticks ); }
      
    private:
      Position childPosition ( const Position &x );

      GLfloat width_;
      GLfloat height_; 
      Widget *child_;
      Color bgColor_;
    };

  } // namespace Ash

} // namespace Dune

#endif // #ifndef DUNE_ASH_WIDGET_SPLIT_HH
