/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

// vim: set expandtab ts=2 sw=2 sts=2:
#ifndef DUNE_ASH_WIDGET_SLIDER_HH
#define DUNE_ASH_WIDGET_SLIDER_HH

#include <dune/ash/fvector.hh>
#include <dune/ash/opengl/opengl.hh>
#include <dune/ash/widget/widget.hh>

namespace Dune
{

  namespace Ash
  {

    // SliderWidget 
    // ------------

    template< Orientation orientation >
    class SliderWidget
    : public Dune::Ash::Widget
    {
      typedef SliderWidget ThisType;

      static const int direction = (orientation == Horizontal ? 0 : 1);

    public:
      typedef FieldVector< GLfloat, 3 > Color;
      typedef std::pair< std::size_t, std::size_t > Range;

      SliderWidget ( const Range &range, const Position &border, GLfloat barWidth = 0.5f );

      void draw ();

      void mouseMove ( const Position &x, const Position &dx, Ticks ticks );

      void mouseButtonDown ( const Position &x, Ticks ticks );

      void mouseButtonUp ( const Position &x, Ticks ticks );

      std::size_t position () const { return position_; }

      void incPosition () { if( !grabbed_ && (position_ < active_.second) ) ++position_; }
      void decPosition () { if( !grabbed_ && (position_ > active_.first) ) --position_; }

      void setPosition ( std::size_t position ) { if( !grabbed_ ) position_ = clamp( position ); }
      void setActiveRange ( const Range &range );

    private:
      void followMouse ( const Position &x );
      std::size_t clamp ( std::size_t position ) const;

      Range range_, active_;
      std::size_t position_;
      bool grabbed_;
      Position border_;
      GLfloat barWidth_;
      Color colorBackground_, colorForeground_, colorInactive_, colorActive_;
    };

  } // namespace Ash

} // namespace Dune

#endif // #ifndef DUNE_ASH_WIDGET_SLIDER_HH
