/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

#ifndef DUNE_ASH_WIDGET_SPLIT_HH
#define DUNE_ASH_WIDGET_SPLIT_HH

#include <dune/ash/opengl/opengl.hh>
#include <dune/ash/widget/widget.hh>

namespace Dune
{

  namespace Ash
  {

    // SplitWidget
    // -----------

    template< Orientation orientation >
    class SplitWidget
    : public Widget
    {
      typedef Widget Base;

      static const int direction = (orientation == Horizontal ? 0 : 1);

    public:
      SplitWidget ( GLdouble leftWeight, GLdouble rightWeight );

      void draw ();

      void mouseMove ( const Position &x, const Position &dx, Ticks ticks );

      void mouseButtonDown ( const Position &x, Ticks ticks );
      void mouseButtonUp ( const Position &x, Ticks ticks );

      void setLeftChild ( Widget *child ) { left_ = child; }
      void setRightChild ( Widget *child ) { right_ = child; }

      void tick ( Ticks ticks );
      
    private:
      void draw ( Widget *child, GLdouble left, GLdouble right );

      static Position childPosition ( GLdouble left, GLdouble right, const Position &x );

      static Position leftChildPosition ( GLdouble ratio, const Position &x ) { return childPosition( 0.0, ratio, x ); }
      static Position rightChildPosition ( GLdouble ratio, const Position &x ) { return childPosition( ratio, 1.0, x ); }

      GLdouble ratio_;
      Widget *left_, *right_;
      Widget *active_;
      Position (*activeChildPosition_) ( GLdouble ratio, const Position &x );
    };

  } // namespace Ash

} // namespace Dune

#endif // #ifndef DUNE_ASH_WIDGET_SPLIT_HH
