/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

// vim: set expandtab ts=2 sw=2 sts=2:
#ifndef DUNE_ASH_WIDGET_TEXT_HH
#define DUNE_ASH_WIDGET_TEXT_HH

#include <dune/ash/fvector.hh>
#include <dune/ash/text/truetypefont.hh>
#include <dune/ash/opengl/opengl.hh>
#include <dune/ash/opengl/displaylist.hh>
#include <dune/ash/opengl/texture.hh>
#include <dune/ash/widget/widget.hh>

namespace Dune
{

  namespace Ash
  {

    enum Alignment { AlignLeft, AlignCenter, AlignRight };

    class TextWidget
    : public Dune::Ash::Widget
    {
      typedef TextWidget ThisType;

    public:
      typedef FieldVector< GLfloat, 3 > Color;

      TextWidget ( const TrueTypeFont &font, GLfloat width, const std::string &text, Alignment alignment = AlignLeft, GLfloat alpha = 1.0f );

      void draw ( );

      void setText ( const std::string &text );
      void setText ( const std::string &text, Alignment alignment );

      const Texture &getTexture () const { return texture_; }

    private:
      const TrueTypeFont &font_;
      Texture texture_;
      GLfloat width_;
      GLfloat alpha_;
      Alignment alignment_;
      Color color_;
      Color background_;
      GLfloat x0_, x1_;
      Rectangle rect_;
    };

  } // namespace Ash

} // namespace Dune

#endif // #ifndef DUNE_ASH_WIDGET_TEXT_HH
