/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

#ifndef DUNE_ASH_WIDGET_WIDGET_HH
#define DUNE_ASH_WIDGET_WIDGET_HH

#include <dune/common/fvector.hh>

namespace Dune
{

  namespace Ash
  {

    // Orientation
    // -----------

    enum Orientation { Horizontal, Vertical };


    // Reference in CCW order
    // ----------------------
    enum Reference { Center,
                     Top,
                     TopLeft, 
                     Left, 
                     BottomLeft, 
                     Bottom, 
                     BottomRight, 
                     Right,
                     TopRight };


    // Widget
    // ------

    class Widget
    {
      typedef Widget ThisType;

    public:
      typedef unsigned long long Ticks;
      typedef FieldVector< double, 2 > Position;

      virtual ~Widget () {}

      virtual void draw () = 0;

      virtual void mouseMove ( const Position &x, const Position &dx, Ticks ticks ) {}

      virtual void mouseButtonDown ( const Position &x, Ticks ticks ) {}
      virtual void mouseButtonUp ( const Position &x, Ticks ticks ) {}

      virtual void tick ( Ticks ticks ) {}
    };

  } // namespace Ash

} // namespace Dune

#endif // #ifndef DUNE_ASH_WIDGET_WIDGET_HH
