/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

// vim: set expandtab ts=2 sw=2 sts=2:
#ifndef DUNE_WRITEUTIL_HH
#define DUNE_WRITEUTIL_HH

#include <dune/fem/io/io.hh>
#include <dune/fem/io/parameter.hh>

#include "../../dune/fem/serialoutputmap.hh"
#include <dune/fem/io/file/vtkio.hh>


inline std::string makeTpFilename ( const std::string filename, const int counter ) 
{
  std::ostringstream newFileName;
  newFileName << filename << "_" << counter;
  return newFileName.str();
}

/**
 * bool willWrite: can be called in baseevolution by eocDataOutput.willWrite( tp )
 *
 */
template < class DiscreteFunctionType, class GridPartType >
void xdrVtkWriteAndRead ( const GridPartType& gridPart, 
                          const std::string& basefilename, 
                          DiscreteFunctionType& solution,
                          bool willWrite, 
                          const int timeStep )
{
  // type of outPutMap 
  typedef Dune::Fem::SerialOutputMap< DiscreteFunctionType > OutputMap;
  OutputMap map( solution );

  if ( true/*willWrite*/ )
  {
    std::string XDRFilename = makeTpFilename ( basefilename, timeStep );
    std::cout << "------->writing XDR output to " << XDRFilename << std::endl;
    map.write( XDRFilename );
    //test if write and read functions work fine:
    Dune::VTKIO< GridPartType, true > vtkio( gridPart, 2 );
    map.read( XDRFilename );
    DiscreteFunctionType sol2= map.solution();
    vtkio.addVertexData ( sol2 );
    std::cout << "-------> test vtk file to testfilevtk" << std::endl;
    std::string VTKFilename = makeTpFilename ( "testvtk", timeStep );
    vtkio.write ( VTKFilename );
	}
}

/**
 * write concentration data into xdr file
 * 
 */
template < class DiscreteFunctionType >
void writeSerialXdr ( const std::string &prefix, const std::string &basefilename, 
                      DiscreteFunctionType& solution,
                      const int counter, const bool verbose = false )
{
  if( verbose )
    std::cout << "in writeXerialXdr( '" << prefix << "', '" << basefilename << "',...);\n";

  if( !Dune::directoryExists( prefix ) ) 
  { 
    Dune::createDirectory( prefix );
  }

  typedef Dune::Fem::SerialOutputMap< DiscreteFunctionType > OutputMap;
  OutputMap map( solution );
  std::string XDRFilename = makeTpFilename( basefilename, counter );
  if( verbose )
    std::cout << "------->writing XDR output to " << ( !prefix.empty()  ? prefix : "./" ) << "/" << XDRFilename << std::endl;
  map.write( prefix, XDRFilename );
}


#endif // #ifndef WRITE_UTIL
