/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

// vim: set expandtab ts=2 sw=2 sts=2:
#include <config.h>

#ifndef SYSCONFDIR
#error "Missing preprocessor define: SYSCONFDIR"
#endif // #ifndef SYSCONFDIR

#ifndef BINDIR
#error "Missing preprocessor define: BINDIR"
#endif // #infdef BINDIR

#include "filemanager.hh"

namespace Ash
{

  // Static Variables
  // ----------------

  static std::string glueScript;



  // Implementation of FileManager
  // -----------------------------

  void FileManager::appendParameterFile ( const std::string &fileName )
  {
    std::vector< std::string > candidates;
    if( !Dune::Parameter::exists( "ash.configpath" ) )
    {
      candidates.push_back( std::string( "." ) );
      candidates.push_back( std::string( "./config" ) );
      candidates.push_back( std::string( SYSCONFDIR ) );
    }
    else
      candidates.push_back( Dune::Parameter::getValue< std::string >( "ash.configpath" ) );

    typedef std::vector< std::string >::const_iterator Iterator;
    for( Iterator it = candidates.begin(); it != candidates.end(); ++it )
    {
      std::string absFileName = *it + "/" + fileName;
      if( Dune::fileExists( absFileName ) )
        return Dune::Parameter::append( absFileName );
    }
  }


  std::string FileManager::macroGridFile ()
  {
    return dataPath() + "/" + Dune::Parameter::getValue< std::string >( "ash.macrogrid" );
  }


  std::string FileManager::dataPath ()
  {
    return Dune::Parameter::getValue< std::string >( "ash.datapath" );
  }


  std::string FileManager::glueScript ()
  {
    if( !Ash::glueScript.empty() )
      return Ash::glueScript;

    std::vector< std::string > candidates;
    if( !Dune::Parameter::exists( "ash.glue" ) )
    {
      candidates.push_back( std::string( "./dune-ash-glue" ) );
      candidates.push_back( std::string( SYSCONFDIR ) + "/dune-ash-glue" );
      candidates.push_back( std::string( BINDIR ) + "/dune-ash-glue" );
    }
    else
      candidates.push_back( Dune::Parameter::getValue< std::string >( "ash.glue" ) );

    typedef std::vector< std::string >::const_iterator Iterator;
    for( Iterator it = candidates.begin(); it != candidates.end(); ++it )
    {
      if( Dune::fileExists( *it ) )
        return (Ash::glueScript = *it);
    }

    std::cerr << "Error: Unable to locate glue script." << std::endl;
    std::cerr << "       Candidates are:" << std::endl;
    for( Iterator it = candidates.begin(); it != candidates.end(); ++it )
      std::cerr << "       " << *it << std::endl;
    abort();
  }

} // namespace Ash
