/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

// vim: set expandtab ts=2 sw=2 sts=2:
#ifndef ASH_COMMON_FILEMANAGER_HH
#define ASH_COMMON_FILEMANAGER_HH

#include <string>

#include <dune/fem/io/io.hh>
#include <dune/fem/io/parameter.hh>

namespace Ash
{

  /*
   * This class manages all data files requiered by Ash. Our primary use for this class are
   * images and parameter files. All paths to data files can be set in a file called
   * 'dune-ash.paths.params'. This file is lookes for in "." and in some other generic
   * directories...
   */
  struct FileManager
  {
    // Returns the absolute path of 'basename' in the image directory. If the image directory
    // is e.g. /foo/bar and the image is by_buzz.png, this returns /foo/bar/by_buzz.png...
    static std::string absoluteImageName ( const std::string &basename )
    {
      return dataPath() + "/images/" + basename;
    }

    static std::string absoluteTextureName ( const std::string &basename )
    {
      return dataPath() + "/textures/" + basename;
    }

    // appends the parameter file 'paramFile' from the parameter file directory
    static void appendParameterFile ( const std::string &fileName );

    // Returns the absolute path of the macro grid file.
    static std::string macroGridFile ();

    static std::string dataPath ();
    static std::string glueScript ();
  };

} // namespace Ash

#endif // #ifndef ASH_COMMON_FILEMANAGER_HH
