/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

#ifndef COMPUTEWINDFIELD_HH
#define COMPUTEWINDFIELD_HH

#include <dune/common/nullptr.hh>

#include <dune/ash/parallel/thread.hh>
#include <dune/ash/widget/event.hh>
#include <dune/ash/widget/grid.hh>
#include <dune/ash/widget/shrink.hh>
#include <dune/ash/widget/floatingtext.hh>

#include "frontendmode.hh"
#include "fileparser.hh"

// ComputeWindFieldThread
// ----------------------

struct ComputeWindFieldThread
: public Dune::Ash::Thread
{
  explicit ComputeWindFieldThread ( const std::string &session );

protected:
  void main ();

private:
  std::string command_;
};



// ComputeWindFieldMode
// --------------------

class ComputeWindFieldMode
: public FrontendMode
{
  typedef ComputeWindFieldMode ThisType;
  typedef FrontendMode BaseType;

public:
  ComputeWindFieldMode ( FrontendModeStack &modeStack, Dune::Ash::Event *eventDone = nullptr );
  ~ComputeWindFieldMode ();

  void initializeMode ();

  void draw ();

  void mouseMove ( const Position &x, const Position &dx, Ticks ticks ) { shrinkWidget_.mouseMove( x, dx, ticks ); }

  void mouseButtonDown ( const Position &x, Ticks ticks ) { shrinkWidget_.mouseButtonDown( x, ticks ); }
  void mouseButtonUp ( const Position &x, Ticks ticks ) { shrinkWidget_.mouseButtonUp( x, ticks ); }

  void tick ( Ticks ticks );

  void setLanguage ( const std::string &language );

private:
  ComputeWindFieldMode ( const ComputeWindFieldMode& );
  ComputeWindFieldMode& operator= ( const ComputeWindFieldMode& );

  ComputeWindFieldThread thread_;
  Dune::Ash::Event *eventDone_;
  FrontendMode *father_;
  Dune::Ash::ShrinkWidget<> shrinkWidget_;
  Dune::Ash::GridWidget gridWidget_;
  Dune::Ash::FloatingTextWidget text_;

  FileParser parser_;
  FileParser::TextMap textMap_;
};

#endif // #ifndef COMPUTEWINDFIELD_HH
