/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

// vim: set expandtab ts=2 sw=2 sts=2:
#ifndef DIFFUSIONMODE_HH
#define DIFFUSIONMODE_HH

#include <dune/ash/widget/multiplesplit.hh>
#include <dune/ash/widget/slider.hh>
#include <dune/ash/widget/shrink.hh>
#include <dune/ash/widget/grid.hh>
#include <dune/ash/widget/text.hh>
#include <dune/ash/widget/floatingtext.hh>

#include "panel.hh"
#include "frontendmode.hh"
#include "visualizationwidget.hh"
#include "sharedinfo.hh"
#include "fileparser.hh"

// DiffusionMode
// -------------

class DiffusionMode
: public FrontendMode
{
  typedef DiffusionMode ThisType;
  typedef FrontendMode BaseType;

  double diffusion ( std::size_t sliderPos ) const  
  {
    double a = std::log( diffMax_ / diffMin_ ) / 100 ;
    return diffMin_ * std::exp( a * sliderPos );
  }

  struct EventNextMode;
  struct EventWindFieldResolution;
  struct EventDisplayGrid;

public:
  DiffusionMode ( FrontendModeStack &modeStack );
  ~DiffusionMode ();

  void cleanUp ();

  void draw () { split_.draw(); }

  void mouseMove ( const Position &x, const Position &dx, Ticks ticks ) { split_.mouseMove( x, dx, ticks ); }

  void mouseButtonDown ( const Position &x, Ticks ticks ) { split_.mouseButtonDown( x, ticks ); }
  void mouseButtonUp ( const Position &x, Ticks ticks ) { split_.mouseButtonUp( x, ticks ); }

  void tick ( Ticks ticks );

  void setLanguage ( const std::string &language );

private:
  DiffusionMode ( const ThisType & );
  const ThisType& operator= ( const ThisType & );

  Dune::Ash::TrueTypeFont *font_;
  Dune::Ash::SplitWidget< Dune::Ash::Horizontal > split_;
  Dune::Ash::SplitWidget< Dune::Ash::Vertical > extraSplit_;
  VisualizationWidget visualizationWidget_;
  Panel panel_;
  Dune::Ash::SliderWidget< Dune::Ash::Horizontal > slider_;
  Dune::Ash::ShrinkWidget<> textBar_;
  Dune::Ash::GridWidget textGrid_;
  Dune::Ash::TextWidget coefficient_;
  Dune::Ash::FloatingTextWidget right_, left_;

  SharedInformation *sharedInformation_;
  FileParser parser_;
  FileParser::TextMap textMap_;

  double diffMin_; 
  double diffMax_; 
};

#endif // #ifndef DIFFUSIONMODE_HH
