/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

#ifndef FILEPARSER_HH
#define FILEPARSER_HH

#include <string>
#include <fstream>
#include <map>

#include <dune/common/nullptr.hh>

#include <dune/fem/io/parameter.hh>

#include "../common/filemanager.hh"
#include "frontendmode.hh"

struct FileParser
{
  typedef std::map< std::string, std::string > TextMap;

  FileParser ( const std::string &fileName )
  : fileName_( fileName )
  {}

  void operator() ( const std::string &language, TextMap &container ) const
  {
    std::stringstream fileName;
    fileName << Ash::FileManager::dataPath() << "/text/" << fileName_ << "." << language;
    std::string absFilename = fileName.str();

    if( !Dune::fileExists( absFilename ) )
    {
      std::size_t pos = absFilename.find_last_of( language );
      absFilename.replace( pos-language.size()+1, language.size(), "en" );
    }

    std::ifstream file( absFilename.c_str() );

    container.clear();
    std::string preamble;
    std::string *content = &preamble;

    while( file )
    {
      std::string line;
      std::getline( file, line);

      if( line.substr(0, 2) == "%[" )
      {
        std::size_t pos = line.find_first_of( ']', 2 );
        if (pos == line.npos )
          return; // throw 

        std::size_t begin = line.find_first_not_of( ' ', 2 );
        std::size_t end = line.find_last_not_of( ' ', pos - 1 );

        const std::string section = line.substr( begin, end - begin + 1 );
        content = &(container[ section ]);
        if( content->empty() )
          (*content) = preamble;
      }
      else
        (*content) += line + "\n";
    }
  }

private:
  const std::string fileName_;
};

#endif // #ifndef FILEPARSER_HH
