/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

#ifndef INFOMODE_HH
#define INFOMODE_HH

#include <list>
#include <string>

#include <dune/common/nullptr.hh>

#include <dune/ash/widget/button.hh>
#include <dune/ash/widget/collection.hh>
#include <dune/ash/widget/floatingtext.hh>
#include <dune/ash/widget/shrink.hh>

#include "frontendmode.hh"



// InfoMode
// --------

class InfoMode
: public FrontendMode
{
  typedef InfoMode ThisType;
  typedef FrontendMode BaseType;

  typedef FrontendModeStack::InfoMap InfoMap;
  typedef InfoMap::const_iterator InfoIterator;

  struct EventClose;
  struct EventBack;
  struct EventLink;

public:
  InfoMode ( FrontendModeStack &modeStack, const std::string &panel = std::string( "maininfo" ) ); // alternative way to get an default parameter from the file might be better
  ~InfoMode ();

  void initializeMode ();

  void draw ();

  void mouseMove ( const Position &x, const Position &dx, Ticks ticks ) { shrinkWidget_.mouseMove( x, dx, ticks ); }

  void mouseButtonDown ( const Position &x, Ticks ticks ) { shrinkWidget_.mouseButtonDown( x, ticks ); }
  void mouseButtonUp ( const Position &x, Ticks ticks ) { shrinkWidget_.mouseButtonUp( x, ticks ); }

  void tick ( Ticks ticks );

  void setLanguage ( const std::string &language );

  void setPanel( const std::string &panel );

private:
  InfoMode ( const InfoMode& );
  InfoMode& operator= ( const InfoMode& );

  const InfoMap &infoMap () const { return modeStack().infoMap(); }

  FrontendMode *father_;
  Dune::Ash::ShrinkWidget<> shrinkWidget_;
  Dune::Ash::CollectionWidget collection_;
  Dune::Ash::ButtonWidget closeButton_, backButton_;
  Dune::Ash::FloatingTextWidget floatingText_;
  std::string curPanel_;
  std::list< std::string > history_;
};

#endif // #ifndef INFOMODE_HH
