/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

// vim: set expandtab ts=2 sw=2 sts=2:
#ifndef INPUTMODE_HH
#define INPUTMODE_HH

#include <list>
#include <utility>

#include <dune/common/fvector.hh>

#include <dune/ash/widget/split.hh>

#include "frontendmode.hh"
#include "inputwidget.hh"
#include "sharedinfo.hh"
#include "panel.hh"


// InputMode
// ---------

class InputMode
: public FrontendMode
{
  typedef InputMode ThisType;
  typedef FrontendMode BaseType;

  struct UndoLastMove;
  struct ComputeWindField;
  struct ClearMousePath;
  struct WindFieldComputed;

public:
  InputMode ( FrontendModeStack &modeStack );
  ~InputMode ();

  void cleanUp ();

  void startSimulation ();

  void draw () { splitWidget_.draw(); }

  void mouseMove ( const Position &x, const Position &dx, Ticks ticks ) { splitWidget_.mouseMove( x, dx, ticks ); }

  void mouseButtonDown ( const Position &x, Ticks ticks ) { splitWidget_.mouseButtonDown( x, ticks ); }
  void mouseButtonUp ( const Position &x, Ticks ticks ) { splitWidget_.mouseButtonUp( x, ticks ); }

  void tick ( Ticks ticks ) { splitWidget_.tick( ticks ); }

  void setLanguage ( const std::string &language );

private:
  Dune::Ash::SplitWidget< Dune::Ash::Horizontal > splitWidget_;
  InputWidget inputWidget_;
  Panel panel_;
  double diffusion_coef_;
  double perturbation_coef_;
  bool field_written_;
  SharedInformation *sharedInformation_;
};

#endif // INPUTMODE_HH
