/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

// vim: set expandtab ts=2 sw=2 sts=2:
#ifndef PANEL_HH
#define PANEL_HH

#include <dune/ash/widget/button.hh>
#include <dune/ash/widget/grid.hh>
#include <dune/ash/widget/multiplesplit.hh>
#include <dune/ash/widget/slider.hh>
#include <dune/ash/widget/widget.hh>
#include <dune/ash/widget/floatingtext.hh>

#include "titlewidget.hh"
#include "fileparser.hh"


// Panel
// -----

class Panel
: public Dune::Ash::Widget
{
  typedef Panel ThisType;
  
  typedef FrontendModeStack::InfoMap InfoMap;
  typedef InfoMap::const_iterator InfoIterator;

  struct EventBack;
  struct EventLink;

public:
  enum ModeId { IdMainMode = 0, IdVolcanoMode = 1, IdInputMode = 2, IdDiffusionMode = 3, IdVisMode = 4 };

  Panel ( FrontendModeStack &modeStack, ModeId modeId, int extraHeight = 0 );
  ~Panel ();

  void draw () { split_.draw(); }

  void mouseMove ( const Position &x, const Position &dx, Ticks ticks ) { split_.mouseMove( x, dx, ticks ); }

  void mouseButtonDown ( const Position &x, Ticks ticks ) { split_.mouseButtonDown( x, ticks ); }
  void mouseButtonUp ( const Position &x, Ticks ticks ) { split_.mouseButtonUp( x, ticks ); }

  void tick ( Ticks ticks );

  void setLanguage ( const std::string &language );

  // Button an slider sets
  Dune::Ash::Widget* setButton ( int pos, Dune::Ash::Widget* button );
  Dune::Ash::Widget* getButton ( int pos );
  void setExtraContent ( Dune::Ash::Widget *extraContent );

  Dune::Ash::Event* setTextEvent ( const std::string &id, Dune::Ash::Event* event );

private:
  // prohibit copying and assignment
  Panel ( const ThisType & );
  const ThisType &operator= ( const ThisType & );

  const InfoMap &infoMap () const { return modeStack_.infoMap(); }

  FrontendModeStack &modeStack_;
  Dune::Ash::MultipleSplitWidget< Dune::Ash::Vertical > split_;
  Dune::Ash::GridWidget bottomButtonArray_;
  Dune::Ash::FloatingTextWidget text_;

  TitleWidget title_;
  FileParser parser_;

  FileParser::TextMap textMap_;
  std::size_t modeId_;
};

#endif // #ifndef PANEL_HH
