/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

// vim: set expandtab ts=2 sw=2 sts=2:
#ifndef VISUALIZATIONMODE_HH
#define VISUALIZATIONMODE_HH

#include <vector>
#include <utility>
#include <cmath>

#include <GL/gl.h>

#include <dune/ash/opengl/displaylist.hh>
#include <dune/ash/parallel/thread.hh>
#include <dune/ash/widget/slider.hh>
#include <dune/ash/widget/split.hh>

#include "frontendmode.hh"
#include "sharedinfo.hh"
#include "panel.hh"
#include "visualizationwidget.hh"


/*
 * class VisualizationMode
 */
class VisualizationMode
: public FrontendMode
{
  typedef VisualizationMode ThisType;
  typedef FrontendMode BaseType;

  struct EventDisplayGrid;
  struct EventWindFieldResolution;
  struct EventPlayBack;
  struct EventStop;

  struct EventRestart;

  struct PullThread;

public:
  VisualizationMode ( FrontendModeStack &modeStack );

  ~VisualizationMode ();

  void initializeMode ();
  void cleanUp ();

  void draw ();

  void mouseMove ( const Position &x, const Position &dx, Ticks ticks ) { splitWidget_.mouseMove( x, dx, ticks ); }

  void mouseButtonDown ( const Position &x, Ticks ticks ) { splitWidget_.mouseButtonDown( x, ticks ); }
  void mouseButtonUp ( const Position &x, Ticks ticks ) { splitWidget_.mouseButtonUp( x, ticks ); }

  void tick ( Ticks ticks ) { splitWidget_.tick( ticks );  }

  void setLanguage ( const std::string &language );

private:
  // forbidden methods
  VisualizationMode ( const VisualizationMode& );
  VisualizationMode& operator= ( const VisualizationMode& );

  void startPullThread ();

  void startBackend ();

  void loadPendingAshSample ();

  void tooglePlayPause ( bool notToPause = false );

  /*
   * private data members
   */
  Dune::Ash::SplitWidget< Dune::Ash::Horizontal > splitWidget_;
  VisualizationWidget visualizationWidget_;
  Panel panel_;
  Dune::Ash::SliderWidget< Dune::Ash::Horizontal > slider_;

  double tickCounter_, ticksPerSecond_, framesPerSecond_;

  SharedInformation *sharedInformation_;

  PullThread *pullThread_;

  std::size_t currentAshSample_;
  bool tickSamples_;

  bool backendStarted_;
};

#endif // VISUALIZATIONMODE_HH
