/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

// vim: set expandtab ts=2 sw=2 sts=2:
#ifndef VISUALIZATIONWIDGET_HH
#define VISUALIZATIONWIDGET_HH

#include <dune/ash/opengl/texture.hh>

#include "volcanowidget.hh"

#include "../common/discretizationtraits.hh"


// VisualizationWidget
// -------------------

class VisualizationWidget
: public VolcanoDisplayWidget
{
  typedef VisualizationWidget ThisType;
  typedef VolcanoDisplayWidget BaseType;

public:
  typedef DiscretizationTraits::GridPartType GridPartType;
  typedef DiscretizationTraits::SolutionVisualizerType SolutionVisualizerType;
  typedef DiscretizationTraits::WindFieldVisualizerType WindFieldVisualizerType;

  explicit VisualizationWidget ( const Dune::Ash::Texture &mapTexture,
                                 const Dune::Ash::BillBoard &iconVolcano );

  void draw ();

  void showGridPart ( const GridPartType *gridPart, double warpFactor = 1.0 )
  {
    gridPart_ = gridPart;
    gridWarpFactor_ = warpFactor;
  }

  void showSolution ( const Dune::Ash::Texture *solutionTexture );
  void showWindField ( const WindFieldVisualizerType *windFieldVisualizer ) { windFieldVisualizer_ = windFieldVisualizer; }

private:
  // prohibit copying and assignment
  VisualizationWidget ( const ThisType & );
  const ThisType &operator= ( const ThisType & );

  const GridPartType *gridPart_;
  const WindFieldVisualizerType *windFieldVisualizer_;
  const Dune::Ash::Texture *solutionTexture_;
  Dune::Ash::DisplayList solutionDisplayList_;
  double gridWarpFactor_;
  double vectorFieldColorScaling_;
};

#endif // #ifndef VISUALIZATIONWIDGET_HH
