/******************************************************************************

  dune-ash is a numerical simulation of the propagation of an ash cloud
  caused by a volcano eruption at a constant height in a given wind field.
  It was motivated by the eruptions of the Icelandic volcanoes in 2010 and
  2011, both leading to the closure of air space above several European
  countries.

  Copyright (C) 2012 - 2013 Janick Gerstenberger
  Copyright (C) 2011        Robert Kloefkorn
  Copyright (C) 2011 - 2013 Tobias Malkmus
  Copyright (C) 2011        David Nies
  Copyright (C) 2011 - 2013 Martin Nolte
  Copyright (C) 2011        Axel Pfeiffer
  Copyright (C) 2011 - 2012 Theresa Strauch

  dune-ash is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, either version 2 of
  the License, or (at your option) any later version.

  dune-ash is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this module. If not, see <http://www.gnu.org/licenses/>.

******************************************************************************/

// vim: set expandtab ts=2 sw=2 sts=2:
#ifndef VOLCANOWIDGET_HH
#define VOLCANOWIDGET_HH

#include <dune/ash/fvector.hh>
#include <dune/ash/opengl/displaylist.hh>
#include <dune/ash/opengl/texture.hh>
#include <dune/ash/widget/display.hh>
#include <dune/ash/widget/grid.hh>



// VolcanoDisplayWidget
// --------------------

class VolcanoDisplayWidget
: public Dune::Ash::DisplayWidget
{
  typedef VolcanoDisplayWidget ThisType;
  typedef Dune::Ash::DisplayWidget BaseType;

public:
  explicit VolcanoDisplayWidget ( const Dune::Ash::Texture &mapTexture,
                                  const Dune::Ash::BillBoard &iconVolcano );

  void draw ();

  void setVolcanoPosition ( const Position &position ) { posVolcano_ = position; }
  const Position &volcanoPosition () const { return posVolcano_; }

private:
  Dune::Ash::DisplayList iconVolcano_;
  Position posVolcano_;
};



// VolcanoWidget
// -------------

class VolcanoWidget
: public VolcanoDisplayWidget
{
  typedef VolcanoWidget ThisType;
  typedef VolcanoDisplayWidget BaseType;

public:
  explicit VolcanoWidget ( const Dune::Ash::Texture &mapTexture,
                           const Dune::Ash::BillBoard &iconVolcano );

  void mouseMove ( const Position &x, const Position &dx, Ticks ticks );

  void mouseButtonDown ( const Position &x, Ticks ticks );
  void mouseButtonUp ( const Position &x, Ticks ticks );

private:
  bool mouseButtonDown_;
};

#endif // #ifndef VOLCANOWIDGET_HH
